% powerSpec.m
% examples to explore the power spectrum of time varying signals, including
% artificial signals generated from sine waves and natural sounds such as
% music, speech, and bird song

clear all
close all

% choose signal to explore
isignal = 1;    % 1 for sine waves
                % 2 for Handel
                % 3 for speech
                % 4 for bird song
                
window = 512;   % time-window size in samples for sliding window spectrogram
noverlap = 256; % overlap between adjacent windows

switch isignal
    
    case 1 % construct a signal from sine waves and play the signal as sound
        dt = 0.001;             % time step in seconds
        fs = 1/dt;              % sampling frequency (default for sound.m is 8192)
        tend = 2;               % play tone for this many seconds
        tvals = 0:dt:tend;      % sequence of time values in seconds
        ftone = 220;            % frequency of sine wave in Hz
        y = sin(2*pi*ftone*tvals);    % construct a single sine wave
        % y = y + 2*sin(pi*ftone*tvals) + 5*sin((2/3)*pi*ftone*tvals);
        sound(y,fs)             % play signal as sound
        
        % compute the power spectrum for the signal - Pxx is the power spectrum,
        % F is a vector of frequencies at which the power spectrum is estimated

        [Pxx,F] = pwelch(y,window,noverlap,[],fs);
        
        % plot the original signal and power spectrum
        figure(1)
        subplot(2,1,1)
        plot(tvals,y)
        xlabel('Time (s)'), ylabel('Amplitude'), title('Original Signal')
        subplot(2,1,2)
        plot(F,Pxx)
        xlim([F(1) F(end)])
        xlabel('Frequency (Hz)'), ylabel('Power'), title('Power Spectrum')
        
        % display the spectrogram: power spectrum as a function of time
        figure(2)
        [s,f,t,p] = spectrogram(y,window,noverlap,[],fs);
        imagesc(t,f,p)
        colorbar
        xlabel('Time (s)'), ylabel('Frequency (Hz)'), title('Spectrogram, Original Signal')
        
    case 2 % load and play a segment of the Hallelujah chorus from Handel's Messiah
        load handel     % includes a different sampling frequency Fs = 8192 Hz
        y_music = y;
        fs_music = Fs;
        sound(y_music,fs_music)
        
        % plot music signal and power spectrum
        [Pxx_music,F_music] = pwelch(y_music,window,noverlap,[],fs_music);
        figure(3)
        subplot(2,1,1)
        t_music = (1:length(y_music))/fs_music;
        plot(t_music,y_music)
        xlabel('Time (s)'), ylabel('Amplitude'), title('Music Signal')
        subplot(2,1,2)
        plot(F_music,Pxx_music)
        xlim([0 2000])
        xlabel('Frequency (Hz)'), ylabel('Power'), title('Power Spectrum, Music')
        
        % display a spectrogram for the music signal
        figure(4)
        [sm,fm,tm,pm] = spectrogram(y_music,window,noverlap,[],fs_music);
        imagesc(tm,fm,pm)
        ylim([0 2000])
        colorbar
        xlabel('Time (s)'), ylabel('Frequency (Hz)'), title('Spectrogram, Music')
        
    case 3 % load and play a speech segment
        [y_speech,fs_speech] = audioread('niceWork.wav');
        y_speech = y_speech(:,1);
        sound(y_speech,fs_speech)
        
        % plot speech signal and power spectrum
        [Pxx_speech,F_speech] = pwelch(y_speech,window,noverlap,[],fs_speech);
        figure(5)
        subplot(2,1,1)
        t_speech = (1:length(y_speech))/fs_speech;
        plot(t_speech,y_speech)
        xlabel('Time (s)'), ylabel('Amplitude'), title('Speech Signal')
        subplot(2,1,2)
        plot(F_speech,Pxx_speech)
        xlim([0 5000])
        xlabel('Frequency (Hz)'), ylabel('Power'), title('Power Spectrum, Speech')
        
        % display a spectrogram for the speech signal
        figure(6)
        [ss,fs,ts,ps] = spectrogram(y_speech,window,noverlap,[],fs_speech);
        imagesc(ts,fs,ps)
        ylim([0 5000])
        colorbar
        xlabel('Time (s)'), ylabel('Frequency (Hz)'), title('Spectrogram, Speech')
        
    case 4 % load and play a bird song segment
        [y_bird,fs_bird] = audioread('song1.wav');
        sound(y_bird,fs_bird)
        
        % plot bird song signal and power spectrum
        [Pxx_bird,F_bird] = pwelch(y_bird,window,noverlap,[],fs_bird);
        figure(7)
        subplot(2,1,1)
        t_bird = (1:length(y_bird))/fs_bird;
        plot(t_bird,y_bird)
        xlabel('Time (s)'), ylabel('Amplitude'), title('Bird Song Signal')
        subplot(2,1,2)
        plot(F_bird,Pxx_bird)
        xlim([0 5000])
        xlabel('Frequency (Hz)'), ylabel('Power'), title('Power Spectrum, Bird Song')
        
        % display a spectrogram for the bird song signal
        figure(8)
        [sb,fb,tb,pb] = spectrogram(y_bird,window,noverlap,[],fs_bird);
        imagesc(tb,fb,pb)
        ylim([0 5000])
        colorbar
        xlabel('Time (s)'), ylabel('Frequency (Hz)'), title('Spectrogram, Bird Song')
        
end % switch for different signals

