% ratSpec.m
% applies spectral analysis tools to real neural data recorded from
% behaving rats in the Wiest lab
% waits for key press (b = back, q = quit, otherwise next trial)

% steps through trials in 2-lfp example LOB session 2 from 25 session set
% and shows frontal/parietal lfp, spectrograms for each, and pre/post-beep 
% coherence spectra on each trial  

clear all
close all

load('B1trials.mat')
% row 1 = frontal, row 2 = parietal elec
% elec x trials x time-points
trials = trialsB1;      % trials matrix lfp x trials x time points
chits = chitsB1;        % indices of 'clean' hit trials 
cmiss = cmissB1;        % indices of clean miss trials
fname = fnameB1;        % data file name

logflag = 0;            % = 1 means spectrograms will plot log scale; = 0 means linear scale

fs = 1000;              % sampling frequency (samples per second)
dt = 1/fs;              % time step between samples
ntrials = size(trials,2);   % number of trials in original data
pre = -1;               % secs before event included in trial
post = 1.0;             % secs after event included in trial
ntpts = size(trials,3);
tax = pre:dt:post;      % time samples
ctrials = sort( [ chits cmiss ] );  % 'clean' hit and miss trials

nselect = length(selected_trialsB1);    % number of selected trials
i = 1;                  % index of next trial to view
while (i <= nselect)
    disp(['trial index ' num2str(i)])   % display trial index
    itrial = selected_trialsB1(i);      % actual number of selected trial
    
    sig1 = squeeze(trials(1, itrial, :));    % frontal electrode single trial
    sig2 = squeeze(trials(2, itrial, :));    % parietal electrode single trial
    h = figure(1);
    subplot(2,1,1)
    plot(tax,sig2)      % plot lfp data
    hold on
    plot(tax,sig1+max(sig2)+abs(min(sig1)),'r');    % note offset for clear viewing
    xlabel('Time (s)')
    ylabel('LFP (mV)')
    % legend('front','pariet')
    if find(chits == itrial)
        trialtype = 'HIT';
    else
        trialtype = 'MISS';
    end
    title(['TRIAL ' num2str(itrial) ': ' trialtype])
    hold off   
    
    subplot(2,2,3)      % display pre-beep coherence spectrum
    mscohere(sig1(1:floor(end/2)),sig2(1:floor(end/2)),[],[],[],fs)
    %     [Cxy,F] = mscohere(X,Y,WINDOW,NOVERLAP,NFFT,Fs) returns the magnitude
    %     squared coherence estimate computed as a function of physical frequency
    %     in Hz. Fs is the sampling frequency specified in Hz.
    title('PRE-BEEP')
    xlabel('f (Hz)')
    ylabel('Coherence')
    xlim([0 30])
    ylim([0 1])
  
    subplot(2,2,4)      % display post-beep coherence spectrum
    mscohere(sig1(floor(end/2)+1:end),sig2(floor(end/2)+1:end),[],[],[],fs)
    title('POST-BEEP')
    xlabel('f (Hz)')
    ylabel('Coherence')
    ylim([0 1])
    xlim([0 30])
    
    figure(2) %____________SPECTROGRAMS____________
    subplot(2,1,1)
    window = 400;
    noverlap = 350;
    f = 2:1:30;     % frequency range and bins to use for spectrogram
    [s,f,t,p] = spectrogram(sig1,window,noverlap,f,fs);
    %     [S,F,T] = spectrogram(X,WINDOW,NOVERLAP,F,Fs) where F is a vector of
    %     frequencies in Hz (with 2 or more elements) computes the spectrogram at
    %     those frequencies using the Goertzel algorithm. The specified
    %     frequencies in F are rounded to the nearest DFT bin commensurate with
    %     the signal's resolution.
    if logflag      % set logflag above
        p = log(p);
    end         
    imagesc(t,f,p,[0 3e-4])
    colorbar
    ylabel('Frequency (Hz)')
    title('Spectrograms: top=frontal, bottom=parietal')
    
    subplot(2,1,2)
    [s,f,t,p] = spectrogram(sig2,window,noverlap,f,fs);
    if logflag      % set logflag above
        p = log(p);
    end 
    imagesc(t,f,p,[0 0.3e-2])
    colorbar
    xlabel('Time (s)')
    ylabel('Frequency (Hz)')
   
    figure(1)
    pause
    % wait for next key press (b = back, q = quit, otherwise next)
    reply = get(h, 'CurrentCharacter');
    if (reply == 'b')
        i = max(1,i-1);
    elseif (reply == 'q')
        i = nselect+1;
    else
        i = i+1;
    end
    
end % trials loops






